package de.eknori;

import lotus.domino.Database;
import lotus.domino.Document;
import lotus.domino.NotesException;
import lotus.domino.NotesFactory;
import lotus.domino.Session;
import lotus.notes.addins.JavaServerAddin;

public class Domino12Test extends JavaServerAddin {
	final String			JADDIN_NAME				= "Domino12Test";
	final String			JADDIN_VERSION			= "2";
	final String			JADDIN_DATE				= "2020-10-22 11:00 CET";
	final String 			DB_FILEPATH = "d12test";

	// Instance variables
	String[] 				args 					= null;
	private int				secondsElapsed			= 30;
	private Session 		session					= null;
	private Database 		db						= null;

	// we expect our first parameter is dedicated for secondsElapsed
	public Domino12Test(String[] args) {
		this.secondsElapsed = Integer.parseInt(args[0]);
	}

	// constructor if no parameters
	public Domino12Test() {}

	@SuppressWarnings("deprecation")
	public void runNotes() {
		displayLoadInfo();

		try {
			session = NotesFactory.createSession();
			String server = session.getServerName();

			while (this.addInRunning()) {

				OSPreemptOccasionally();
				if (this.AddInHasSecondsElapsed(secondsElapsed)) {
					db = session.getDatabase(server, DB_FILEPATH);
					long count = db.getAllDocuments().getCount();
					logMessage("DocCount: " + Long.toString(count));
					
					db.transactionBegin();
					Document doc = db.createDocument();
					doc.replaceItemValue("Form", "commit");
					doc.replaceItemValue("Subject", "CommitTest");
					doc.save();
					db.transactionCommit();
					
					db.transactionBegin();
					doc = db.createDocument();
					doc.replaceItemValue("Form", "rollback");
					doc.replaceItemValue("Subject", "RollbackTest");
					doc.save();
					db.transactionRollback();
					
					doc.recycle();
					db.recycle();
				}
			}
		} catch(Exception e) {
			e.printStackTrace();
		}

		terminate();
	}

	/**
	 * Display run configuration
	 */
	private void displayLoadInfo() {
		String jvmVersion = System.getProperty("java.specification.version", "0");

		logMessage("version		" + this.JADDIN_VERSION);
		logMessage("build date		" + this.JADDIN_DATE);
		logMessage("java		" + jvmVersion);
		logMessage("seconds elapsed	" + secondsElapsed);
	}

	/**
	 * Write a log message to the Domino console. The message string will be prefixed with the add-in name
	 * followed by a column, e.g. <code>"AddinName: xxxxxxxx"</code>
	 * 
	 * @param	message		Message to be displayed
	 */
	private final void logMessage(String message) {
		AddInLogMessageText(this.JADDIN_NAME + ": " + message, 0);
	}

	/**
	 * This method is called by the Java runtime during garbage collection.
	 */
	public void finalize() {
		terminate();

		super.finalize();
	}

	/**
	 * Terminate all variables
	 */
	private void terminate() {
		try {
			if (this.db != null) {
				this.db.recycle();
			}
			if (this.session != null) {
				this.session.recycle();
			}

			logMessage("UNLOADED (OK)");
		} catch (NotesException e) {
			logMessage("UNLOADED (**FAILED**)");
		}
	}

}